@@ -3,19 +3,20 @@ TODO:
     - look into https://github.com/click-contrib/click-help-colors
 """
 
+from csv import DictReader
 import datetime
+from distutils.util import strtobool
 import json
+import re
 import sys
+from typing import Dict, Generator, Iterable, List, Optional, Tuple, Union
 import urllib.parse
 import uuid
-from csv import DictReader
-from distutils.util import strtobool
-from typing import Dict, Generator, Iterable, List, Optional, Tuple, Union
 
 import click
 from globus_sdk import TransferClient
 
-from timer_cli.auth import get_authorizer_for_scope, get_current_user
+from timer_cli.auth import get_current_user
 from timer_cli.auth import logout as auth_logout
 from timer_cli.auth import revoke_login
 from timer_cli.job import (
@@ -27,6 +28,11 @@ from timer_cli.job import (
     show_job_list,
 )
 from timer_cli.output import make_table
+from timer_cli.transfer import (
+    TRANSFER_ALL_SCOPE,
+    error_if_not_activated,
+    get_transfer_client,
+)
 
 # List of datetime formats accepted as input. (`%z` means timezone.)
 DATETIME_FORMATS = [
@@ -38,12 +44,34 @@ DATETIME_FORMATS = [
 ]
 
 
-TRANSFER_ALL_SCOPE = "urn:globus:auth:scope:transfer.api.globus.org:all"
-
 # TODO: make configurable/environment/maybe even check terminal width?
 MAX_CONTENT_WIDTH = 100
 
 
+timedelta_regex = re.compile(
+    r"\s*((?P<weeks>\d+)w)?"
+    r"\s*((?P<days>\d+)d)?"
+    r"\s*((?P<hours>\d+)h)?"
+    r"\s*((?P<minutes>\d+)m)?"
+    r"\s*((?P<seconds>\d+)s?)?"
+)
+
+
+INTERVAL_HELP = (
+    "Interval at which the job should run. Use 'w', 'd', 'h', 'm', and 's' as suffixes"
+    " to specify weeks, days, hours, minutes, and seconds. Examples: '1h 30m', '500s',"
+    " '24h', '1d 12h', '2w', etc. Must be in order: hours -> minutes -> seconds. You"
+    " should either use quotes ('1d 2h') or write without spaces (1d2h)."
+)
+
+
+def _parse_timedelta(s: str) -> datetime.timedelta:
+    groups = {k: int(v) for k, v in timedelta_regex.match(s).groupdict(0).items()}
+    # timedelta accepts kwargs for units up through days, have to convert weeks
+    groups["days"] += groups.pop("weeks", 0) * 7
+    return datetime.timedelta(**groups)
+
+
 def _un_parse_opt(opt: str):
     return "--" + opt.replace("_", "-")
 
@@ -78,13 +106,8 @@ def _read_csv(
             yield {k: transform_val(k, v) for k, v in row_dict.items()}
 
 
-def _get_transfer_client():
-    transfer_authorizer = get_authorizer_for_scope(TRANSFER_ALL_SCOPE, all_scopes=[])
-    return TransferClient(transfer_authorizer)
-
-
 def _get_required_data_access_scopes(collection_ids: Iterable[str]) -> List[str]:
-    tc = _get_transfer_client()
+    tc = get_transfer_client()
     data_access_scopes: List[str] = []
     for collection_id in collection_ids:
         collection_id_info = tc.get_endpoint(collection_id)
@@ -117,14 +140,16 @@ def show_usage(cmd: click.Command):
     the click context.
     """
     ctx = click.get_current_context()
-    ctx.max_content_width = MAX_CONTENT_WIDTH
+    # TODO: disabling this next line for the time being because of inconsistent
+    # behavior between this function and calling --help directly, which would produce
+    # different output. still have to figure that out
+    # ctx.max_content_width = MAX_CONTENT_WIDTH
     formatter = ctx.make_formatter()
     cmd.format_help_text(ctx, formatter)
     cmd.format_options(ctx, formatter)
     cmd.format_epilog(ctx, formatter)
     click.echo(formatter.getvalue().rstrip("\n"))
-    ctx.exit()
-    sys.exit(2)
+    ctx.exit(2)
 
 
 class Command(click.Command):
@@ -273,8 +298,8 @@ def job():
 @click.option(
     "--interval",
     required=True,
-    type=int,
-    help="Interval in seconds at which the job should run",
+    type=str,
+    help=INTERVAL_HELP,
 )
 @click.option(
     "--scope",
@@ -322,7 +347,7 @@ def job():
 def submit(
     name: str,
     start: Optional[datetime.datetime],
-    interval: int,
+    interval: str,
     scope: str,
     action_url: urllib.parse.ParseResult,
     action_body: Optional[str],
@@ -332,10 +357,15 @@ def submit(
     """
     Submit a job.
     """
+    interval_seconds = _parse_timedelta(interval).total_seconds()
+    if not interval_seconds:
+        raise click.UsageError(f"Couldn't parse interval: {interval}")
+    if interval_seconds < 60:
+        raise click.UsageError(f"Interval is too short, minimum is 1 minute")
     response = job_submit(
         name,
         start,
-        interval,
+        interval_seconds,
         scope,
         action_url,
         action_body,
@@ -393,7 +423,9 @@ def list(show_deleted: bool, verbose: bool):
     help="Show full JSON output",
 )
 @click.argument("job_id", type=uuid.UUID, required=False)
-def status(job_id: Optional[uuid.UUID], show_deleted: bool, verbose: bool, show_all: bool):
+def status(
+    job_id: Optional[uuid.UUID], show_deleted: bool, verbose: bool, show_all: bool
+):
     """
     Return the status of the job with the given ID.
 
@@ -405,10 +437,12 @@ def status(job_id: Optional[uuid.UUID], show_deleted: bool, verbose: bool, show_
     CHECK THE --verbose OUTPUT TO BE CERTAIN YOUR TRANSFERS ARE WORKING.
     """
     if not job_id and not show_all:
-        click.echo("Error: must provide either a job ID or the --all option\n", err=True)
+        click.echo(
+            "Error: must provide either a job ID or the --all option\n", err=True
+        )
         show_usage(click.get_current_context().command)
     if show_all:
-        show_job_list(job_list(), as_table=False)
+        show_job_list(job_list(), as_table=False, verbose=verbose)
     else:
         show_job(job_status(job_id, show_deleted=show_deleted), verbose=verbose)
 
@@ -420,9 +454,14 @@ def status(job_id: Optional[uuid.UUID], show_deleted: bool, verbose: bool, show_
     is_flag=True,
     help="Show full JSON output",
 )
-@click.argument("job_id", type=uuid.UUID)
-def delete(job_id: uuid.UUID, verbose: bool):
-    show_job(job_delete(job_id), verbose=verbose, was_deleted=True)
+@click.argument("job_ids", type=uuid.UUID, nargs=-1)
+def delete(job_ids: Iterable[uuid.UUID], verbose: bool):
+    start = True
+    for job_id in job_ids:
+        if not start:
+            click.echo("")
+        show_job(job_delete(job_id), verbose=verbose, was_deleted=True)
+        start = False
 
 
 @job.command(cls=Command)
@@ -441,8 +480,8 @@ def delete(job_id: uuid.UUID, verbose: bool):
 @click.option(
     "--interval",
     required=True,
-    type=int,
-    help="Interval in seconds at which the job should run",
+    type=str,
+    help=INTERVAL_HELP,
 )
 @click.option(
     "--source-endpoint",
@@ -465,6 +504,18 @@ def delete(job_id: uuid.UUID, verbose: bool):
         " contain only letters/numbers/spaces, and the following characters: - _ ,"
     ),
 )
+@click.option(
+    "--stop-after-date",
+    required=False,
+    type=click.DateTime(formats=DATETIME_FORMATS),
+    help=("Stop running the transfer after this date"),
+)
+@click.option(
+    "--stop-after-runs",
+    required=False,
+    type=int,
+    help=("Stop running the transfer after this number of runs have happened"),
+)
 @click.option(
     "--sync-level",
     required=False,
@@ -479,6 +530,30 @@ def delete(job_id: uuid.UUID, verbose: bool):
         " source and destination do not match."
     ),
 )
+@click.option(
+    "--encrypt-data",
+    is_flag=True,
+    default=False,
+    help="Whether Transfer should encrypt data sent through the network using TLS",
+)
+@click.option(
+    "--verify-checksum",
+    is_flag=True,
+    default=False,
+    help=(
+        "Whether Transfer should verify file checksums and retry if the source and"
+        " destination don't match"
+    ),
+)
+@click.option(
+    "--preserve-timestamp",
+    is_flag=True,
+    default=False,
+    help=(
+        "Whether Transfer should set file timestamps on the destination to match the"
+        " origin"
+    ),
+)
 @click.option(
     "--item",
     "-i",
@@ -512,11 +587,16 @@ def delete(job_id: uuid.UUID, verbose: bool):
 def transfer(
     name: str,
     start: Optional[datetime.datetime],
-    interval: int,
+    interval: str,
     source_endpoint: str,
     dest_endpoint: str,
     label: Optional[str],
+    stop_after_date: Optional[datetime.datetime],
+    stop_after_runs: Optional[int],
     sync_level: Optional[int],
+    encrypt_data: bool,
+    verify_checksum: bool,
+    preserve_timestamp: bool,
     item: Optional[List[Tuple[str, str, Optional[str]]]],
     items_file: Optional[str],
     verbose: bool,
@@ -528,9 +608,9 @@ def transfer(
     action_url = urllib.parse.urlparse(
         "https://actions.automate.globus.org/transfer/transfer/run"
     )
-    data_access_scopes = _get_required_data_access_scopes(
-        [source_endpoint, dest_endpoint]
-    )
+    endpoints = [source_endpoint, dest_endpoint]
+    error_if_not_activated(endpoints)
+    data_access_scopes = _get_required_data_access_scopes(endpoints)
     transfer_ap_scope = (
         "https://auth.globus.org/scopes/actions.globus.org/transfer/transfer"
     )
@@ -561,19 +641,29 @@ def transfer(
     if label:
         action_body["label"] = label
     else:
-        action_body["label"] = f"From Timer service job named {name}"
-    if sync_level:
+        action_body["label"] = f"Job from Timer service named {name}"
+    if sync_level is not None:
         action_body["sync_level"] = sync_level
+    action_body["encrypt_data"] = encrypt_data
+    action_body["verify_checksum"] = verify_checksum
+    action_body["preserve_timestamp"] = preserve_timestamp
     callback_body = {"body": action_body}
+    interval_seconds = _parse_timedelta(interval).total_seconds()
+    if not interval_seconds:
+        raise click.UsageError(f"Couldn't parse interval: {interval}")
+    if interval_seconds < 60:
+        raise click.UsageError(f"Interval is too short, minimum is 1 minute")
     response = job_submit(
         name,
         start,
-        interval,
+        interval_seconds,
         transfer_ap_scope,
         action_url,
         action_body=None,
         action_file=None,
         callback_body=callback_body,
+        stop_after_date=stop_after_date,
+        stop_after_runs=stop_after_runs,
     )
     show_job(response, verbose=verbose)
 
@@ -605,7 +695,7 @@ def whoami(format: str):
         click.echo(
             "Not logged in yet; use `globus-timer session login` to initialize your"
             " session",
-            err=True
+            err=True,
         )
         sys.exit(1)
     full_fields = ["name", "email", "preferred_username", "organization"]
